// Datei KlammernPruefenA.java
/* ------------------------------------------------------------------------
Dieses Programm erwartet einen Kommandozeilen-Parameter, und zwar den
Pfadnamen einer Textdatei. Die Datei wird daraufhin geprueft, ob alle
runden, eckigen und geschweiften Klammern richtig gepaart sind und gibt
das Ergebnis der Pruefung zur Standardausgabe aus.

Beispiele fuer Klammern, die NICHT richtig gepaart sind:
Beispiel1: [ ... { ... } [ ...        Es fehlen ]]
Beispiel2: ( ... [ ... ) ...          ')' ohne passende '('
Beispiel3: )(                         ')' ohne passende '('
------------------------------------------------------------------------ */
import java.io.FileReader;
import java.io.BufferedReader;
import java.util.Stack;
import java.util.EmptyStackException;

public class KlammernPruefenA {
   // ------------------------------------------------------------------
   static private void melde_1() {
      // Erfolgsmeldung:
      pln("Alle Klammern sind richtig gepaart!");
   } // melde_1
   // ------------------------------------------------------------------
   static private void melde_2(char klammer, int zeilenNr, int spaltenNr) {
      // Fehlermeldung (fuer Faelle wie Beispiel2 und 3):
      printf("Falsche Klammer %c in Zeile %d, Spalte %d%n",
         klammer, zeilenNr, spaltenNr);
   } // melde_2
   // ------------------------------------------------------------------
   static private void melde_3(Stack<Character> stapel)  {
      // Fehlermeldung (fuer Faelle wie Beispiel1):
      p("Am Ende fehlen Klammern: ");
      while(! stapel.empty()) {p(stapel.pop());}
      pln();
   } // melde_3
   // ------------------------------------------------------------------
   static public void main(String[] args) throws java.io.IOException {

      // Ein Stapel fuer die noch ungepaarten Klammern:
      Stack<Character> stapel = new Stack<Character>();

      String zeile;            // Die aktuelle (gerade eingelesene) Zeile
      char   aktZeichen = '?'; // Das aktuelle Zeichen der aktuellen Zeile

      int    zeilenNr   = 0;
      int    spaltenNr  = 0;

      // Abstrakte Namen wie AUF1 und ZU1 fuer die Klammern sind aenderungs-
      // freundlicher als konkrete Namen wie RUNDE_AUF und RUNDE_ZU.
      final char AUF1 = '(', ZU1 = ')'; // Klammer-Art 1 auf und zu
      final char AUF2 = '[', ZU2 = ']'; // Klammer-Art 2 auf und zu
      final char AUF3 = '{', ZU3 = '}'; // Klammer-Art 3 auf und zu

      // Erwartet wird genau ein Kommandozeilen-Parameter:
//       if (args.length != 1) {
//          pln("------------------------------------------------------");
//          pln("Beispiel fuer einen richtigen Aufruf dieses Programms:");
//          pln(                                                        );
//          pln("> java KlammernPruefenA Klammern3.ein\n"               );
//          pln(                                                        );
//          pln("Wirkung: Die Datei Klammern3.ein wird geprueft."       );
//          pln("------------------------------------------------------");
//          return; // Programm wird beendet
//       }

      // Im fertigen Programm soll fr so vereinbart werden:
//    FileReader     fr = new FileReader(args[0]);
      // Fuer die Programmentwicklung ist folgende Vereinbarung bequemer,
      // weil man dann keinen Kommandozeilen-Parameter angeben muss:
      FileReader     fr = new FileReader("Klammern1.ein");
      BufferedReader br = new BufferedReader(fr);

      // ******************************************************************
      // Der folgende try-catch-Block liest eine Datei zeilenweise ein,
      // und gibt die einzelnen Zeilen mit einer Zeilen-Nummer davor
      // zur Standardausgabe aus.
      //
      // Sie sollen den try-catch-Block so aendern, dass er die Datei
      // daraufhin prueft, ob alle runden, eckigen und geschweiften
      // Klammern "richtig gepaart" sind.
      try {
         // Alle Zeilen und Zeichen der Textdatei verarbeiten
         while (true) {
            zeile = br.readLine();
            if (zeile == null) break; // Ende der Datei erreicht
            zeilenNr++;

            printf("%3d: %s%n", zeilenNr, zeile);
         } // while


      } // try
      catch (EmptyStackException aus) {
         printf("%s%n", aus);
      }

      finally {
         fr.close();
      }
      // ******************************************************************

   } // main
   // ------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ------------------------------------------------------------------
} // class KlammernPruefenA
