// Datei Tripel.java
/* ------------------------------------------------------------------------
Jedes Objekt dieser Klasse ("jedes Tripel") enthaelt
3 oeffentliche, unveraenderbare     int-Variablen namens A, B und C und
5 oeffentliche, unveraenderbare boolean-Variablen namens
IST_DREISEIT, IST_GLEICHSEITIG, IST_GLEICHSCHENKLIG, IST_SCHIEF und
IST_RECHTWINKLIG.

Die int-Variablen A, B und C haben nicht-negative Werte (der einzige
Konstruktor ersetzt negative Parameter durch 0).

Die boolen-Variablen haben den Wert true, wenn die entsprechende
Bedingungen erfuellt ist:
-------------------+------------------------------------------------------+
boolean-Variable   | Bedingung                                            |
-------------------+------------------------------------------------------+
IST_DREISEIT       | Die drei Dreiseit-Ungleichungen                      |
                   | A<=B+C, B<=C+A und C<=A+B sind erfuellt.             |
-------------------+------------------------------------------------------+
IST_GLEICHSEITIG   | Alle drei int-Variablen A, B, C haben gleiche Werte. |
-------------------+------------------------------------------------------+
IST_GLEICHSCHENKLIG| Mindestens zwei der int-Variablen haben gleiche Werte|
-------------------+------------------------------------------------------+
IST_SCHIEF         | Die drei int-Variablen haben paarweise verschiedene  |
                   | Werte (oder: Keine zwei haben gleiche Werte).        |
-------------------+------------------------------------------------------+
IST_RECHTWINKLIG   | Die drei int-Variablen erfuellen den Satz des        |
                   | Pythagoras und sind nicht alle drei gleich 0.        |
-------------------+------------------------------------------------------+
*/

public class Tripel {
   // ---------------------------------------------------------------------
   // Die int-Komponenten eines Tripels:
   public final int     A;
   public final int     B;
   public final int     C;

   // Die boolean-Komponenten eines Tripels (beschreiben wichtige
   // Eigenschaften des Tripels):
   public final boolean IST_DREISEIT;
   public final boolean IST_GLEICHSEITIG;
   public final boolean IST_GLEICHSCHENKLIG;
   public final boolean IST_SCHIEF;
   public final boolean IST_RECHTWINKLIG;
   // ---------------------------------------------------------------------
   // Ein Konstruktor, mit dem jedes neue Tripel-Objekt initialisiert wird:
   public Tripel(final int pa, final int pb, final int pc) {

      // Negative Parameter werden durch 0 ersetzt:
      A = Math.max(0, pa);
      B = Math.max(0, pb);
      C = Math.max(0, pc);

      // Ersetzen Sie jeweils "false" durch "den richtigen boolean-Ausdruck",
      // der dort hingehoert:
      IST_DREISEIT        = false; // false SOLLTE ERSETZT WERDEN
      IST_GLEICHSEITIG    = false; // false SOLLTE ERSETZT WERDEN
      IST_GLEICHSCHENKLIG = false; // false SOLLTE ERSETZT WERDEN
      IST_SCHIEF          = false; // false SOLLTE ERSETZT WERDEN
      IST_RECHTWINKLIG    = false; // false SOLLTE ERSETZT WERDEN
   } // Konstruktor Tripel
   // ---------------------------------------------------------------------
   public String toString() {
      // Liefert eine String-Representation des aktuellen Tripel-Objekts.
      // Dieser String enthaelt u.a. 6 Zeilenwechsel-Zeichen '\n'.
      return
      "(A: " + A + ", B: " + B + ", C: " + C + ")"  + "\n" +
      "Ist ein Dreiseit:    " + IST_DREISEIT        + "\n" +
      "Ist gleichseitig:    " + IST_GLEICHSEITIG    + "\n" +
      "Ist gleichschenklig: " + IST_GLEICHSCHENKLIG + "\n" +
      "Ist schief:          " + IST_SCHIEF          + "\n" +
      "Ist rechtwinklig:    " + IST_RECHTWINKLIG    + "\n" ;
   } // toString
   // ---------------------------------------------------------------------
   static public void main(String[] _) {
      // Ein "Mini-Test" der Klasse Tripel (ein Tripel-Objekt wird
      // erzeugt und seine String-Repraesentation wird ausgegeben):
      printf("Tripel: Jetzt geht es los!%n");
      printf("----------------------------------%n");

      Tripel t01 = new Tripel(1, 2, 3);
      printf("t01: %s", t01.toString());

      printf("----------------------------------%n");
      printf("Tripel: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Tripel
/* ------------------------------------------------------------------------
Ausgabe:

Tripel: Jetzt geht es los!
----------------------------------
t01: (A: 1, B: 2, C: 3)
Ist ein Dreiseit:    true
Ist gleichseitig:    false
Ist gleichschenklig: false
Ist schief:          true
Ist rechtwinklig:    false
----------------------------------
Tripel: Das war's erstmal!
------------------------------------------------------------------------ */
